package com.generalnegentropics.archis.utils;

//
// This code had no license, so I assume it's public domain.  The algorithm
// certainly is.
//

/**
 * Methods for base64 encoding and decoding
 *
 * @author Unknown (found in old code snippet archive via Google)
 * @version 1.0
 */

public class Base64 {
  // Characters used in encoding
  private static char[] alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=".toCharArray();

  // Lookup table
  private static byte[] codes = new byte[256];
  static
  {
    for (int i=0; i<256; i++)
      codes[i] = -1;
    for (int i = 'A'; i <= 'Z'; i++)
      codes[i] = (byte)(     i - 'A');
    for (int i = 'a'; i <= 'z'; i++)
      codes[i] = (byte)(26 + i - 'a');
    for (int i = '0'; i <= '9'; i++)
      codes[i] = (byte)(52 + i - '0');
    codes['+'] = 62;
    codes['/'] = 63;
  }

  /**
   * Encodes a byte[] array into base64
   *
   * @param data byte[] array of data
   * @return Base64-encoded data
   */
  public static char[] encode(byte[] data)
  {
    char[] out = new char[((data.length + 2) / 3) * 4];
    for (int i=0, index=0; i<data.length; i+=3, index+=4) {
      boolean quad = false;
      boolean trip = false;

      int val = (0xFF & (int) data[i]);
      val <<= 8;
      if ((i+1) < data.length) {
        val |= (0xFF & (int) data[i+1]);
        trip = true;
      }
      val <<= 8;
      if ((i+2) < data.length) {
        val |= (0xFF & (int) data[i+2]);
        quad = true;
      }
      out[index+3] = alphabet[(quad? (val & 0x3F): 64)];
      val >>= 6;
      out[index+2] = alphabet[(trip? (val & 0x3F): 64)];
      val >>= 6;
      out[index+1] = alphabet[val & 0x3F];
      val >>= 6;
      out[index+0] = alphabet[val & 0x3F];
    }
    return out;
  }

  /**
   * Encodes a byte[] array into base64
   *
   * @param data byte[] array of data
   * @return Base64-encoded data
   */
  public String encodeString(byte[] data)
  {
    return new String(encode(data));
  }

  /**
   * Decodes a char[] array containing base64 encoded data
   *
   * @param data Base64-encoded data
   * @return Original byte array data
   */
  public static byte[] decode(char[] data)
  {
    int tempLen = data.length;
    for (int ix=0;ix<data.length;ix++) {
      if ((codes[data[ix] & 0xFF] < 0)&&(data[ix] != 61))
        --tempLen;
    }
    int len = ((tempLen + 3) / 4) * 3;
    if ((tempLen > 0)&&(data[tempLen-1] == '='))
      --len;
    if ((tempLen > 1)&&(data[tempLen-2] == '='))
      --len;
    byte[] out = new byte[len];

    int shift = 0;
    int accum = 0;
    int index = 0;

    for (int ix=0; ix<data.length; ix++) {
      int value = codes[data[ix] & 0xFF];
      if (value >= 0) {
        accum <<= 6;
        shift += 6;
        accum |= value;
        if ( shift >= 8 ) {
          shift -= 8;
          out[index++] = (byte)((accum >> shift) & 0xff);
        }
      }
    }

    return out;
  }

  /**
   * Decodes a string containing base64 encoded data
   *
   * @param data Base64-encoded data
   * @return Original byte array data
   */
  public static byte[] decodeString(String data)
  {
    int dataLen = data.length();
    int tempLen = dataLen;
    for (int ix=0;ix<dataLen;ix++) {
      char c = data.charAt(ix);
      if ((codes[c & 0xFF] < 0)&&(c != 61))
        --tempLen;
    }
    int len = ((tempLen + 3) / 4) * 3;
    if ((tempLen > 0)&&(data.charAt(tempLen-1) == '='))
      --len;
    if ((tempLen > 1)&&(data.charAt(tempLen-2) == '='))
      --len;
    byte[] out = new byte[len];

    int shift = 0;
    int accum = 0;
    int index = 0;

    for (int ix=0; ix<dataLen; ix++) {
      int value = codes[data.charAt(ix) & 0xFF];
      if (value >= 0) {
        accum <<= 6;
        shift += 6;
        accum |= value;
        if ( shift >= 8 ) {
          shift -= 8;
          out[index++] = (byte)((accum >> shift) & 0xff);
        }
      }
    }

    return out;
  }
}
