package com.generalnegentropics.archis.utils;

//
// Archis Cellular Artificial Life Simulator
// Copyright (C) 2001-2003  Adam Ierymenko
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

/**
 * An IntegerInput implementation that reads from an int[] buffer.
 *
 * @author Adam Ierymenko
 * @version 1.0
 */

public class BufferedIntegerInput implements IntegerInput
{
  private int[] buf;
  private int ptr;

  /**
   * <p>Constructs a new buffered integer input</p>
   *
   * <p>Note that buf is not copied, and so it should not be modified after
   * this is created.</p>
   *
   * @param buf Buffer to read from
   */
  public BufferedIntegerInput(int[] buf)
  {
    ptr = 0;
    this.buf = buf;
  }

  public int read()
  {
    return ((ptr < buf.length) ? buf[ptr++] : 0);
  }

  /**
   * Gets the buffer that is stored internally (not a copy)
   *
   * @return Internal buffer
   */
  public int[] getBuffer()
  {
    return buf;
  }
}
