package com.generalnegentropics.archis.utils;

//
// Archis Cellular Artificial Life Simulator
// Copyright (C) 2001-2003  Adam Ierymenko
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

/**
 * A random source using Mersenne Twister algorithm
 *
 * @author Adam Ierymenko
 * @version 1.0
 */

public class MersenneTwisterRandomSource implements RandomSource
{
  private MersenneTwister mt;
  private static char[] rclist = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789".toCharArray();

  /**
   * Constructs a new MersenneTwisterRandomSource
   *
   * @param seed Seed value
   */
  public MersenneTwisterRandomSource(long seed)
  {
    mt = new MersenneTwister(seed);
  }

  public int randomInteger()
  {
    synchronized(mt) {
      return mt.next(32);
    }
  }

  public int randomPositiveInteger()
  {
    synchronized(mt) {
      int i = mt.next(32);
      return ((i==-2147483648)?2147483647:((i<0)?-i:i));
    }
  }

  public long randomLong()
  {
    synchronized(mt) {
      return (((long)mt.next(32)) << 32) + mt.next(32);
    }
  }

  public long randomPositiveLong()
  {
    synchronized(mt) {
      long i = (((long)mt.next(32)) << 32) + mt.next(32);
      return ((i==Long.MIN_VALUE)?Long.MAX_VALUE:Math.abs(i));
    }
  }

  public char randomChar()
  {
    synchronized(mt) {
      return mt.nextChar();
    }
  }

  public char randomLetterOrNumber()
  {
    return rclist[randomPositiveInteger()%rclist.length];
  }

  public byte randomByte()
  {
    synchronized(mt) {
      return mt.nextByte();
    }
  }

  public short randomShort()
  {
    synchronized(mt) {
      return mt.nextShort();
    }
  }

  public boolean randomBoolean()
  {
    synchronized(mt) {
      return mt.nextBoolean();
    }
  }

  public boolean randomEvent(float probability)
  {
    synchronized(mt) {
      return (mt.nextFloat()<=probability);
    }
  }

  public boolean randomEvent(double probability)
  {
    synchronized(mt) {
      return (mt.nextDouble()<=probability);
    }
  }

  public double randomDouble()
  {
    synchronized(mt) {
      return mt.nextDouble();
    }
  }

  public float randomFloat()
  {
    synchronized(mt) {
      return mt.nextFloat();
    }
  }
}
